@@ -1,19 +1,12 @@
 import requests
 import json
 import urllib3
-import logging
-import os
-import sys
+from powerprotect import get_module_logger
 
 urllib3.disable_warnings()
 
-logger = logging.getLogger("ppdm")
-logger.setLevel(os.getenv("LOG_LEVEL", "INFO"))
-ch = logging.StreamHandler()
-formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - '
-                              '%(message)s')
-ch.setFormatter(formatter)
-logger.addHandler(ch)
+ppdm_logger = get_module_logger(__name__)
+ppdm_logger.propagate = False
 
 
 """
@@ -32,39 +25,28 @@ class Ppdm:
     def __init__(self, **kwargs):
         """Create a PPDM object that is authenticated"""
         self.server = kwargs['server']
-        self.__password = kwargs['password']
+        self.__password = kwargs.get('password', "")
         self.username = kwargs.get('username', "admin")
+        self.headers = {'Content-Type': 'application/json'}
         self.__token = kwargs.get('token', "")
-        self.headers = ({'Content-Type': 'application/json'}
-                        ).update({'Authorization': self.__token})
-        if not self.__token:
-            self.ppdm_login()
+        if self.__token:
+            self.headers.update({'Authorization': self.__token})
 
-    def ppdm_login(self):
+    def login(self):
         """Login method that extends the headers property to include the
         authorization key/value"""
-        logger.debug("Method: ppdm_login")
+        ppdm_logger.debug("Method: __login")
         body = {"username": self.username, "password": self.__password}
         response = self.__rest_post("/login", body)
-        self.headers.update({'Authorization': response.json()['access_token']})
-
-    def get_protection_rules(self):
-        logger.debug("Method: get_protection_rules")
-        response = self.__rest_get("/protection-rules")
-        return json.loads(response.text)
-
-    def get_protection_rule_by_name(self, name):
-        logger.debug("Method: get_protection_rule_by_name")
-        response = self.__rest_get("/protection-rules"
-                                   f"?filter=name%20eq%20%22{name}%22")
-        return json.loads(response.text)["content"][0]
+        self.headers.update({'Authorization': json.loads(response.text
+                                                         )['access_token']})
 
     def create_protection_rule(self, policy_name, rule_name, inventory_type,
-                               label):
+                               label, **kwargs):
         protection_policy_id = (self.get_protection_policy_by_name(policy_name)
                                 )["content"][0]["id"]
-        logger.debug("Method: create_protection_rule")
-        body = {"action": "MOVE_TO_GROUP",
+        ppdm_logger.debug("Method: create_protection_rule")
+        body = {"action": kwargs.get('action', 'MOVE_TO_GROUP'),
                 "name": rule_name,
                 "actionResult": protection_policy_id,
                 "conditions": [{
@@ -74,7 +56,7 @@ class Ppdm:
                 }],
                 "connditionConnector": "AND",
                 "inventorySourceType": inventory_type,
-                "priority": 1,
+                "priority": kwargs.get('priority', 1),
                 "tenant": {
                     "id": "00000000-0000-4000-a000-000000000000"
                 }
@@ -82,30 +64,47 @@ class Ppdm:
         response = self.__rest_post("/protection-rules", body)
         return json.loads(response.text)
 
+    def get_protection_rules(self):
+        ppdm_logger.debug("Method: get_protection_rules")
+        response = self.__rest_get("/protection-rules")
+        return json.loads(response.text)
+
+    def get_protection_rule_by_name(self, name):
+        ppdm_logger.debug("Method: get_protection_rule_by_name")
+        response = self.__rest_get("/protection-rules"
+                                   f"?filter=name%20eq%20%22{name}%22")
+        return json.loads(response.text)["content"][0]
+
     def update_protection_rule(self, body):
-        logger.debug("Method: update_protection_rule")
+        ppdm_logger.debug("Method: update_protection_rule")
         protection_rule_id = body["id"]
         response = self.__rest_put("/protection-rules"
                                    f"/{protection_rule_id}", body)
         return json.loads(response.text)
 
+    def compare_protection_rule(self, existing_rule_name, expected_rule_body):
+        ppdm_logger.debug("Method: compare_protection_rule")
+        existing_rule = self.get_protection_rule_by_name(existing_rule_name)
+        if self.__compare_body(existing_rule, expected_rule_body):
+            return True
+
     def delete_protection_rule(self, id):
-        logger.debug("Method: delete_protection_rule")
+        ppdm_logger.debug("Method: delete_protection_rule")
         self.__rest_delete(f"/protection-rules/{id}")
 
     def get_protection_policies(self):
-        logger.debug("Method: get_protection_policies")
+        ppdm_logger.debug("Method: get_protection_policies")
         response = self.__rest_get("/protection-policies")
         return json.loads(response.text)
 
     def get_protection_policy_by_name(self, name):
-        logger.debug("Method: get_protection_policy_by_name")
+        ppdm_logger.debug("Method: get_protection_policy_by_name")
         response = self.__rest_get("/protection-policies"
                                    f"?filter=name%20eq%20%22{name}%22")
         return json.loads(response.text)
 
     def __rest_get(self, uri):
-        logger.debug("Method: __rest_get")
+        ppdm_logger.debug("Method: __rest_get")
         response = requests.get(f"https://{self.server}:8443/api/v2"
                                 f"{uri}",
                                 verify=False,
@@ -113,14 +112,14 @@ class Ppdm:
         try:
             response.raise_for_status()
         except requests.exceptions.HTTPError as e:
-            logger.error(f"Response Code: {response.status_code} "
-                         f"Reason: {response.text} "
-                         f"Error: {e}")
-            sys.exit(1)
+            ppdm_logger.error(f"Response Code: {response.status_code} "
+                              f"Reason: {response.text} "
+                              f"Error: {e}")
+            return None
         return response
 
     def __rest_delete(self, uri):
-        logger.debug("Method: __rest_delete")
+        ppdm_logger.debug("Method: __rest_delete")
         response = requests.delete(f"https://{self.server}:8443/api/v2"
                                    f"{uri}",
                                    verify=False,
@@ -128,14 +127,14 @@ class Ppdm:
         try:
             response.raise_for_status()
         except requests.exceptions.HTTPError as e:
-            logger.error(f"Response Code: {response.status_code} "
-                         f"Reason: {response.text} "
-                         f"Error: {e}")
-            sys.exit(1)
+            ppdm_logger.error(f"Response Code: {response.status_code} "
+                              f"Reason: {response.text} "
+                              f"Error: {e}")
+            return None
         return response
 
     def __rest_post(self, uri, body):
-        logger.debug("Method: __rest_post")
+        ppdm_logger.debug("Method: __rest_post")
         response = requests.post(f"https://{self.server}:8443/api/v2"
                                  f"{uri}",
                                  verify=False,
@@ -144,14 +143,19 @@ class Ppdm:
         try:
             response.raise_for_status()
         except requests.exceptions.HTTPError as e:
-            logger.error(f"Response Code: {response.status_code} "
-                         f"Reason: {response.text} "
-                         f"Error: {e}")
-            sys.exit(1)
+            ppdm_logger.error(f"Response Code: {response.status_code} "
+                              f"Reason: {response.text} "
+                              f"Error: {e}")
+            return None
         return response
 
+    def __compare_body(self, server_dict, client_dict):
+        test = {**server_dict, **client_dict}
+        if server_dict == test:
+            return True
+
     def __rest_put(self, uri, body):
-        logger.debug("Method: __rest_put")
+        ppdm_logger.debug("Method: __rest_put")
         response = requests.put(f"https://{self.server}:8443/api/v2"
                                 f"{uri}",
                                 verify=False,
@@ -160,8 +164,8 @@ class Ppdm:
         try:
             response.raise_for_status()
         except requests.exceptions.HTTPError as e:
-            logger.error(f"Response Code: {response.status_code} "
-                         f"Reason: {response.text} "
-                         f"Error: {e}")
-            sys.exit(1)
+            ppdm_logger.error(f"Response Code: {response.status_code} "
+                              f"Reason: {response.text} "
+                              f"Error: {e}")
+            return None
         return response
